@@ -5,7 +5,7 @@
 
   Author: Aleksander Machniak <alec@alec.pl>
 
-  $Id: rcube_sieve.php 3855 2010-08-02 13:23:37Z alec $
+  $Id: rcube_sieve.php 3894 2010-08-12 12:28:01Z alec $
 
 */
 
@@ -339,6 +339,10 @@ class rcube_sieve
                     $content .= "# rule:[".$name[$i]."]\n";
                 }
                 elseif (isset($name[$i])) {
+                    // This preg_replace is added because I've found some Avelsieve scripts
+                    // with rules containing "if" here. I'm not sure it was working
+                    // before without this or not.
+                    $token = preg_replace('/^if\s+/', '', trim($token));
                     $content .= "if $token\n";
                     $i++;
                 }
