@@ -4,8 +4,8 @@
  +-----------------------------------------------------------------------+
  | program/include/rcmail.php                                            |
  |                                                                       |
- | This file is part of the RoundCube Webmail client                     |
- | Copyright (C) 2008-2010, RoundCube Dev. - Switzerland                 |
+ | This file is part of the Roundcube Webmail client                     |
+ | Copyright (C) 2008-2010, Roundcube Dev. - Switzerland                 |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -15,13 +15,13 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: rcmail.php 3875 2010-08-06 14:59:34Z alec $
+ $Id: rcmail.php 3994 2010-09-26 15:06:55Z thomasb $
 
 */
 
 
 /**
- * Application class of RoundCube Webmail
+ * Application class of Roundcube Webmail
  * implemented as singleton
  *
  * @package Core
@@ -45,6 +45,7 @@ class rcmail
   public $comm_path = './';
 
   private $texts;
+  private $books = array();
 
 
   /**
@@ -235,7 +236,6 @@ class rcmail
       $this->db = new rcube_mdb2($config_all['db_dsnw'], $config_all['db_dsnr'], $config_all['db_persistent']);
       $this->db->sqlite_initials = INSTALL_PATH . 'SQL/sqlite.initial.sql';
       $this->db->set_debug((bool)$config_all['sql_debug']);
-      $this->db->db_connect('w');
     }
 
     return $this->db;
@@ -280,6 +280,10 @@ class rcmail
       $contacts = new rcube_contacts($this->db, $this->user->ID);
     }
 
+    // add to the 'books' array for shutdown function
+    if (!in_array($contacts, $this->books))
+      $this->books[] = $contacts;
+
     return $contacts;
   }
 
@@ -607,10 +611,6 @@ class rcmail
       if ($virtuser = rcube_user::email2user($username))
         $username = $virtuser;
 
-    // lowercase username if it's an e-mail address (#1484473)
-    if (strpos($username, '@'))
-      $username = mb_strtolower($username);
-
     // user already registered -> overwrite username
     if ($user = rcube_user::query($username, $host))
       $username = $user->data['username'];
@@ -618,8 +618,16 @@ class rcmail
     if (!$this->imap)
       $this->imap_init();
 
+    // try IMAP login
+    if (!($imap_login = $this->imap->connect($host, $username, $pass, $imap_port, $imap_ssl))) {
+      // lowercase username if it's an e-mail address (#1484473)
+      $username_lc = mb_strtolower($username);
+      if ($username_lc != $username && ($imap_login = $this->imap->connect($host, $username_lc, $pass, $imap_port, $imap_ssl)))
+        $username = $username_lc;
+    }
+
     // exit if IMAP login failed
-    if (!($imap_login  = $this->imap->connect($host, $username, $pass, $imap_port, $imap_ssl)))
+    if (!$imap_login)
       return false;
 
     $this->set_imap_prop();
@@ -642,7 +650,7 @@ class rcmail
       else {
         raise_error(array(
           'code' => 600, 'type' => 'php',
-	      'file' => __FILE__, 'line' => __LINE__,
+          'file' => __FILE__, 'line' => __LINE__,
           'message' => "Failed to create a user record. Maybe aborted by a plugin?"
           ), true, false);
       }
@@ -810,9 +818,9 @@ class rcmail
     if (($attrib['uppercase'] && strtolower($attrib['uppercase']=='first')) || $attrib['ucfirst'])
       return ucfirst($text);
     else if ($attrib['uppercase'])
-      return strtoupper($text);
+      return mb_strtoupper($text);
     else if ($attrib['lowercase'])
-      return strtolower($text);
+      return mb_strtolower($text);
 
     return $text;
   }
@@ -874,7 +882,7 @@ class rcmail
 
       if ($dh = @opendir(INSTALL_PATH . 'program/localization')) {
         while (($name = readdir($dh)) !== false) {
-          if ($name{0}=='.' || !is_dir(INSTALL_PATH . 'program/localization/' . $name))
+          if ($name[0] == '.' || !is_dir(INSTALL_PATH . 'program/localization/' . $name))
             continue;
 
           if ($label = $rcube_languages[$name])
@@ -974,8 +982,9 @@ class rcmail
     if (is_object($this->smtp))
       $this->smtp->disconnect();
 
-    if (is_object($this->contacts))
-      $this->contacts->close();
+    foreach ($this->books as $book)
+      if (is_object($book))
+        $book->close();
 
     // before closing the database connection, write session data
     if ($_SERVER['REMOTE_ADDR'])
@@ -1071,7 +1080,7 @@ class rcmail
     if (function_exists('mcrypt_module_open') &&
         ($td = mcrypt_module_open(MCRYPT_TripleDES, "", MCRYPT_MODE_CBC, "")))
     {
-      $iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
+      $iv = $this->create_iv(mcrypt_enc_get_iv_size($td));
       mcrypt_generic_init($td, $this->config->get_crypto_key($key), $iv);
       $cipher = $iv . mcrypt_generic($td, $clear);
       mcrypt_generic_deinit($td);
@@ -1082,9 +1091,7 @@ class rcmail
 
       if (function_exists('des')) {
         $des_iv_size = 8;
-        $iv = '';
-        for ($i = 0; $i < $des_iv_size; $i++)
-          $iv .= sprintf("%c", mt_rand(0, 255));
+        $iv = $this->create_iv($des_iv_size);
         $cipher = $iv . des($this->config->get_crypto_key($key), $clear, 1, 1, $iv);
       }
       else {
@@ -1153,7 +1160,23 @@ class rcmail
   }
 
   /**
-   * Build a valid URL to this instance of RoundCube
+   * Generates encryption initialization vector (IV)
+   *
+   * @param int Vector size
+   * @return string Vector string
+   */
+  private function create_iv($size)
+  {
+    // mcrypt_create_iv() can be slow when system lacks entrophy
+    // we'll generate IV vector manually
+    $iv = '';
+    for ($i = 0; $i < $size; $i++)
+        $iv .= chr(mt_rand(0, 255));
+    return $iv;
+  }
+
+  /**
+   * Build a valid URL to this instance of Roundcube
    *
    * @param mixed Either a string with the action or url parameters as key-value pairs
    * @return string Valid application URL
