@@ -4,8 +4,8 @@
  +-----------------------------------------------------------------------+
  | program/include/rcube_contacts.php                                    |
  |                                                                       |
- | This file is part of the RoundCube Webmail client                     |
- | Copyright (C) 2006-2010, RoundCube Dev. - Switzerland                 |
+ | This file is part of the Roundcube Webmail client                     |
+ | Copyright (C) 2006-2010, Roundcube Dev. - Switzerland                 |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -15,7 +15,7 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: rcube_contacts.php 3698 2010-06-02 10:23:06Z alec $
+ $Id: rcube_contacts.php 3989 2010-09-25 13:03:53Z alec $
 
 */
 
