@@ -4,8 +4,8 @@
  +-----------------------------------------------------------------------+
  | program/include/rcube_imap_generic.php                                |
  |                                                                       |
- | This file is part of the RoundCube Webmail client                     |
- | Copyright (C) 2005-2010, RoundCube Dev. - Switzerland                 |
+ | This file is part of the Roundcube Webmail client                     |
+ | Copyright (C) 2005-2010, Roundcube Dev. - Switzerland                 |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -21,7 +21,7 @@
  | Author: Ryo Chijiiwa <Ryo@IlohaMail.org>                              |
  +-----------------------------------------------------------------------+
 
- $Id: rcube_imap_generic.php 3866 2010-08-04 13:49:48Z alec $
+ $Id: rcube_imap_generic.php 4048 2010-10-06 08:02:47Z thomasb $
 
 */
 
@@ -119,7 +119,7 @@ class rcube_imap_generic
     {
     }
 
-    private function putLine($string, $endln=true)
+    function putLine($string, $endln=true)
     {
         if (!$this->fp)
             return false;
@@ -128,14 +128,21 @@ class rcube_imap_generic
     		write_log('imap', 'C: '. rtrim($string));
 	    }
 
-        return fputs($this->fp, $string . ($endln ? "\r\n" : ''));
+        $res = fwrite($this->fp, $string . ($endln ? "\r\n" : ''));
+
+   		if ($res === false) {
+           	@fclose($this->fp);
+           	$this->fp = null;
+   		}
+
+        return $res;
     }
 
     // $this->putLine replacement with Command Continuation Requests (RFC3501 7.5) support
-    private function putLineC($string, $endln=true)
+    function putLineC($string, $endln=true)
     {
         if (!$this->fp)
-            return NULL;
+            return false;
 
 	    if ($endln)
 		    $string .= "\r\n";
@@ -166,7 +173,7 @@ class rcube_imap_generic
 	    return $res;
     }
 
-    private function readLine($size=1024)
+    function readLine($size=1024)
     {
 		$line = '';
 
@@ -199,7 +206,7 @@ class rcube_imap_generic
 	    return $line;
     }
 
-    private function multLine($line, $escape=false)
+    function multLine($line, $escape=false)
     {
 	    $line = rtrim($line);
 	    if (preg_match('/\{[0-9]+\}$/', $line)) {
@@ -220,7 +227,7 @@ class rcube_imap_generic
         return $line;
     }
 
-    private function readBytes($bytes)
+    function readBytes($bytes)
     {
 	    $data = '';
 	    $len  = 0;
@@ -242,7 +249,7 @@ class rcube_imap_generic
     }
 
     // don't use it in loops, until you exactly know what you're doing
-    private function readReply(&$untagged=null)
+    function readReply(&$untagged=null)
     {
 	    do {
 		    $line = trim($this->readLine(1024));
@@ -257,7 +264,7 @@ class rcube_imap_generic
 	    return $line;
     }
 
-    private function parseResult($string)
+    function parseResult($string)
     {
 	    $a = explode(' ', trim($string));
 	    if (count($a) >= 2) {
@@ -278,7 +285,7 @@ class rcube_imap_generic
     }
 
     // check if $string starts with $match (or * BYE/BAD)
-    private function startsWith($string, $match, $error=false, $nonempty=false)
+    function startsWith($string, $match, $error=false, $nonempty=false)
     {
 	    $len = strlen($match);
 	    if ($len == 0) {
@@ -303,31 +310,6 @@ class rcube_imap_generic
 	    return false;
     }
 
-    private function startsWithI($string, $match, $error=false, $nonempty=false)
-    {
-	    $len = strlen($match);
-	    if ($len == 0) {
-		    return false;
-	    }
-        if (!$this->fp) {
-            return true;
-        }
-	    if (strncasecmp($string, $match, $len) == 0) {
-		    return true;
-	    }
-	    if ($error && preg_match('/^\* (BYE|BAD) /i', $string, $m)) {
-            if (strtoupper($m[1]) == 'BYE') {
-                @fclose($this->fp);
-                $this->fp = null;
-            }
-		    return true;
-	    }
-        if ($nonempty && !strlen($string)) {
-            return true;
-        }
-	    return false;
-    }
-
     function getCapability($name)
     {
 	    if (in_array($name, $this->capability)) {
@@ -346,14 +328,10 @@ class rcube_imap_generic
         }
 	    do {
 		    $line = trim($this->readLine(1024));
-		    $a = explode(' ', $line);
-		    if ($line[0] == '*') {
-			    while (list($k, $w) = each($a)) {
-				    if ($w != '*' && $w != 'CAPABILITY')
-    					$this->capability[] = strtoupper($w);
-			    }
-		    }
-	    } while ($a[0] != 'cp01');
+	        if (preg_match('/^\* CAPABILITY (.+)/i', $line, $matches)) {
+		        $this->capability = explode(' ', strtoupper($matches[1]));
+	        }
+	    } while (!$this->startsWith($line, 'cp01', true));
 
 	    $this->capability_readed = true;
 
@@ -445,23 +423,7 @@ class rcube_imap_generic
 		    return true;
 	    }
 
-        if (!$this->getCapability('NAMESPACE')) {
-	        return false;
-	    }
-
-	    if (!$this->putLine("ns1 NAMESPACE")) {
-            return false;
-        }
-	    do {
-		    $line = $this->readLine(1024);
-		    if ($this->startsWith($line, '* NAMESPACE')) {
-			    $i    = 0;
-			    $line = $this->unEscape($line);
-			    $data = $this->parseNamespace(substr($line,11), $i, 0, 0);
-		    }
-	    } while (!$this->startsWith($line, 'ns1', true, true));
-
-	    if (!is_array($data)) {
+	    if (!is_array($data = $this->_namespace())) {
 	        return false;
 	    }
 
@@ -510,13 +472,9 @@ class rcube_imap_generic
 	    }
 
 	    do {
-		    $line = $this->readLine(500);
-		    if ($line[0] == '*') {
-			    $line = rtrim($line);
-			    $a = rcube_explode_quoted_string(' ', $this->unEscape($line));
-			    if ($a[0] == '*') {
-			        $delimiter = str_replace('"', '', $a[count($a)-2]);
-        		}
+		    $line = $this->readLine(1024);
+		    if (preg_match('/^\* LIST \([^\)]*\) "*([^"]+)"* ""/', $line, $m)) {
+    	        $delimiter = $this->unEscape($m[1]);
 		    }
 	    } while (!$this->startsWith($line, 'ghd', true, true));
 
@@ -526,23 +484,10 @@ class rcube_imap_generic
 
 	    // if that fails, try namespace extension
 	    // try to fetch namespace data
-	    if (!$this->putLine("ns1 NAMESPACE")) {
+	    if (!is_array($data = $this->_namespace())) {
             return false;
         }
 
-	    do {
-		    $line = $this->readLine(1024);
-		    if ($this->startsWith($line, '* NAMESPACE')) {
-			    $i = 0;
-			    $line = $this->unEscape($line);
-			    $data = $this->parseNamespace(substr($line,11), $i, 0, 0);
-		    }
-	    } while (!$this->startsWith($line, 'ns1', true, true));
-
-	    if (!is_array($data)) {
-	        return false;
-	    }
-
 	    // extract user space data (opposed to global/shared space)
 	    $user_space_data = $data[0];
 	    if (!is_array($user_space_data)) {
@@ -561,6 +506,31 @@ class rcube_imap_generic
 	    return $delimiter;
     }
 
+    function _namespace()
+    {
+        if (!$this->getCapability('NAMESPACE')) {
+	        return false;
+	    }
+
+	    if (!$this->putLine("ns1 NAMESPACE")) {
+            return false;
+        }
+
+	    do {
+		    $line = $this->readLine(1024);
+		    if (preg_match('/^\* NAMESPACE/', $line)) {
+			    $i = 0;
+			    $data = $this->parseNamespace(substr($line,11), $i, 0, 0);
+		    }
+	    } while (!$this->startsWith($line, 'ns1', true, true));
+
+	    if (!is_array($data)) {
+	        return false;
+	    }
+
+        return $data;
+    }
+
     function connect($host, $user, $password, $options=null)
     {
 	    // set options
@@ -655,7 +625,7 @@ class rcube_imap_generic
                	$this->putLine("tls0 STARTTLS");
 
 			    $line = $this->readLine(4096);
-                if (!$this->startsWith($line, "tls0 OK")) {
+                if (!preg_match('/^tls0 OK/', $line)) {
 				    $this->error    = "Server responded to STARTTLS with: $line";
 				    $this->errornum = -2;
                     return false;
@@ -730,7 +700,7 @@ class rcube_imap_generic
 
     function close()
     {
-	    if ($this->putLine("I LOGOUT")) {
+	    if ($this->logged && $this->putLine("I LOGOUT")) {
 		    if (!feof($this->fp))
 			    fgets($this->fp, 1024);
 	    }
@@ -749,31 +719,24 @@ class rcube_imap_generic
 
 	    if ($this->putLine("sel1 SELECT \"".$this->escape($mailbox).'"')) {
 		    do {
-			    $line = rtrim($this->readLine(300));
-			    $a = explode(' ', $line);
-			    if (count($a) == 3) {
-				    $token = strtoupper($a[2]);
-				    if ($token == 'EXISTS') {
-					    $this->exists = (int) $a[1];
-				    }
-				    else if ($token == 'RECENT') {
-					    $this->recent = (int) $a[1];
-				    }
+			    $line = rtrim($this->readLine(512));
+
+			    if (preg_match('/^\* ([0-9]+) (EXISTS|RECENT)$/', $line, $m)) {
+			        $token = strtolower($m[2]);
+			        $this->$token = (int) $m[1];
 			    }
 			    else if (preg_match('/\[?PERMANENTFLAGS\s+\(([^\)]+)\)\]/U', $line, $match)) {
 				    $this->permanentflags = explode(' ', $match[1]);
 			    }
 		    } while (!$this->startsWith($line, 'sel1', true, true));
 
-		    if (strcasecmp($a[1], 'OK') == 0) {
+            if ($this->parseResult($line) == 0) {
 			    $this->selected = $mailbox;
 			    return true;
 		    }
-            else {
-                $this->error = "Couldn't select $mailbox";
-            }
 	    }
 
+        $this->error = "Couldn't select $mailbox";
         return false;
     }
 
@@ -831,7 +794,8 @@ class rcube_imap_generic
 		    $add = $this->compressMessageSet(join(',', $add));
 
 	    $command  = "s ".$is_uid."SORT ($field) $encoding ALL";
-	    $line     = $data = '';
+	    $line     = '';
+	    $data     = '';
 
 	    if (!empty($add))
 	        $command .= ' '.$add;
@@ -841,7 +805,7 @@ class rcube_imap_generic
 	    }
 	    do {
 		    $line = rtrim($this->readLine());
-		    if ($this->startsWith($line, '* SORT')) {
+		    if (!$data && preg_match('/^\* SORT/', $line)) {
 			    $data .= substr($line, 7);
     		} else if (preg_match('/^[0-9 ]+$/', $line)) {
 			    $data .= $line;
@@ -1265,7 +1229,7 @@ class rcube_imap_generic
 					    	}
         					break;
 					    case 'in-reply-to':
-			    			$result[$id]->in_reply_to = preg_replace('/[\n<>]/', '', $string);
+			    			$result[$id]->in_reply_to = str_replace(array("\n", '<', '>'), '', $string);
 				    		break;
     					case 'references':
 	    					$result[$id]->references = $string;
@@ -1569,7 +1533,7 @@ class rcube_imap_generic
 	    }
 	    do {
 		    $line = trim($this->readLine());
-		    if ($this->startsWith($line, '* THREAD')) {
+		    if (!$data && preg_match('/^\* THREAD/', $line)) {
     			$data .= substr($line, 9);
 	    	} else if (preg_match('/^[0-9() ]+$/', $line)) {
 		    	$data .= $line;
@@ -1610,7 +1574,7 @@ class rcube_imap_generic
 
     	do {
 	    	$line = trim($this->readLine());
-		    if ($this->startsWith($line, '* SEARCH')) {
+		    if (!$data && preg_match('/^\* SEARCH/', $line)) {
     			$data .= substr($line, 8);
 	    	} else if (preg_match('/^[0-9 ]+$/', $line)) {
 		    	$data .= $line;
@@ -1669,8 +1633,11 @@ class rcube_imap_generic
             $command = 'LIST';
         }
 
+        $ref = $this->escape($ref);
+        $mailbox = $this->escape($mailbox);
+
     	// send command
-	    if (!$this->putLine($key." ".$command." \"". $this->escape($ref) ."\" \"". $this->escape($mailbox) ."\"")) {
+	    if (!$this->putLine($key." ".$command." \"". $ref ."\" \"". $mailbox ."\"")) {
 		    $this->error = "Couldn't send $command command";
 	        return false;
 	    }
@@ -1679,16 +1646,15 @@ class rcube_imap_generic
 	    do {
 		    $line = $this->readLine(500);
 		    $line = $this->multLine($line, true);
-		    $a    = explode(' ', $line);
+		    $line = trim($line);
 
-		    if (($line[0] == '*') && ($a[1] == $command)) {
-			    $line = rtrim($line);
-        		// split one line
-			    $a = rcube_explode_quoted_string(' ', $line);
-        		// last string is folder name
-			    $folders[] = preg_replace(array('/^"/', '/"$/'), '', $this->unEscape($a[count($a)-1]));
-		        // second from last is delimiter
-        		$delim = trim($a[count($a)-2], '"');
+		    if (preg_match('/^\* '.$command.' \(([^\)]*)\) "*([^"]+)"* (.*)$/', $line, $m)) {
+        		// folder name
+   			    $folders[] = preg_replace(array('/^"/', '/"$/'), '', $this->unEscape($m[3]));
+		        // attributes
+//        		$attrib = explode(' ', $this->unEscape($m[1]));
+		        // delimiter
+//        		$delim = $this->unEscape($m[2]);
 		    }
 	    } while (!$this->startsWith($line, $key, true));
 
@@ -1821,6 +1787,10 @@ class rcube_imap_generic
 
         	while ($bytes > 0) {
     		    $line = $this->readLine(1024);
+
+    		    if ($line === NULL)
+    		        break;
+
             	$len  = strlen($line);
 
 		        if ($len > $bytes) {
@@ -2001,7 +1971,7 @@ class rcube_imap_generic
 	    return false;
     }
 
-    function appendFromFile($folder, $path, $headers=null, $separator="\n\n")
+    function appendFromFile($folder, $path, $headers=null)
     {
 	    if (!$folder) {
 	        return false;
@@ -2017,14 +1987,16 @@ class rcube_imap_generic
 		    return false;
 	    }
 
+        $body_separator = "\r\n\r\n";
 	    $len = filesize($path);
+
 	    if (!$len) {
 		    return false;
 	    }
 
         if ($headers) {
             $headers = preg_replace('/[\r\n]+$/', '', $headers);
-            $len += strlen($headers) + strlen($separator);
+            $len += strlen($headers) + strlen($body_separator);
         }
 
     	// send APPEND command
@@ -2040,7 +2012,7 @@ class rcube_imap_generic
 
             // send headers with body separator
             if ($headers) {
-			    $this->putLine($headers . $separator, false);
+			    $this->putLine($headers . $body_separator, false);
             }
 
 		    // send file
@@ -2109,7 +2081,7 @@ class rcube_imap_generic
 	    if ($this->putLine('QUOT1 GETQUOTAROOT "INBOX"')) {
 		    do {
 			    $line = rtrim($this->readLine(5000));
-			    if ($this->startsWith($line, '* QUOTA ')) {
+			    if (preg_match('/^\* QUOTA /', $line)) {
 				    $quota_lines[] = $line;
         		}
 		    } while (!$this->startsWith($line, 'QUOT1', true, true));
@@ -2118,7 +2090,7 @@ class rcube_imap_generic
 	    // return false if not found, parse if found
 	    $min_free = PHP_INT_MAX;
 	    foreach ($quota_lines as $key => $quota_line) {
-		    $quota_line   = preg_replace('/[()]/', '', $quota_line);
+		    $quota_line   = str_replace(array('(', ')'), '', $quota_line);
 		    $parts        = explode(' ', $quota_line);
 		    $storage_part = array_search('STORAGE', $parts);
 
@@ -2193,7 +2165,7 @@ class rcube_imap_generic
 	    $in_quotes = false;
 	    $elem      = 0;
 
-        for ($i;$i<$len;$i++) {
+        for ($i; $i<$len; $i++) {
 		    $c = (string)$str[$i];
 		    if ($c == '(' && !$in_quotes) {
 			    $i++;
@@ -2204,7 +2176,7 @@ class rcube_imap_generic
     		} else if ($c == '\\') {
 			    $i++;
 			    if ($in_quotes) {
-				    $data[$elem] .= $c.$str[$i];
+				    $data[$elem] .= $str[$i];
         		}
 		    } else if ($c == '"') {
 			    $in_quotes = !$in_quotes;
