@@ -4,8 +4,8 @@
  +-----------------------------------------------------------------------+
  | program/include/rcube_json_output.php                                 |
  |                                                                       |
- | This file is part of the RoundCube Webmail client                     |
- | Copyright (C) 2008-2010, RoundCube Dev. - Switzerland                 |
+ | This file is part of the Roundcube Webmail client                     |
+ | Copyright (C) 2008-2010, Roundcube Dev. - Switzerland                 |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -16,7 +16,7 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: rcube_json_output.php 3553 2010-04-23 11:25:37Z alec $
+ $Id: rcube_json_output.php 3989 2010-09-25 13:03:53Z alec $
 
 */
 
@@ -36,16 +36,18 @@ class rcube_json_output
     private $callbacks = array();
     private $message = null;
 
+    public $browser;
     public $type = 'js';
     public $ajax_call = true;
-    
-    
+
+
     /**
      * Constructor
      */
-    public function __construct($task)
+    public function __construct($task=null)
     {
-        $this->config = rcmail::get_instance()->config;
+        $this->config  = rcmail::get_instance()->config;
+        $this->browser = new rcube_browser();
     }
 
 
