@@ -3,8 +3,8 @@
  +-----------------------------------------------------------------------+
  | program/include/rcube_ldap.php                                        |
  |                                                                       |
- | This file is part of the RoundCube Webmail client                     |
- | Copyright (C) 2006-2009, RoundCube Dev. - Switzerland                 |
+ | This file is part of the Roundcube Webmail client                     |
+ | Copyright (C) 2006-2009, Roundcube Dev. - Switzerland                 |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -14,7 +14,7 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: rcube_ldap.php 3780 2010-06-23 09:55:08Z alec $
+ $Id: rcube_ldap.php 3989 2010-09-25 13:03:53Z alec $
 
 */
 
