@@ -4,8 +4,8 @@
  +-----------------------------------------------------------------------+
  | program/include/rcube_message.php                                     |
  |                                                                       |
- | This file is part of the RoundCube Webmail client                     |
- | Copyright (C) 2008-2010, RoundCube Dev. - Switzerland                 |
+ | This file is part of the Roundcube Webmail client                     |
+ | Copyright (C) 2008-2010, Roundcube Dev. - Switzerland                 |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -15,7 +15,7 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: rcube_message.php 3873 2010-08-06 13:22:20Z alec $
+ $Id: rcube_message.php 3999 2010-09-29 09:23:55Z alec $
 
 */
 
@@ -214,23 +214,22 @@ class rcube_message
 
             if ($mimetype == 'text/plain') {
                 $out = $this->imap->get_message_part($this->uid, $mime_id, $part);
-        
+
                 // re-format format=flowed content
                 if ($part->ctype_secondary == 'plain' && $part->ctype_parameters['format'] == 'flowed')
                     $out = self::unfold_flowed($out);
                 break;
             }
             else if ($mimetype == 'text/html') {
-                $html_part = $this->imap->get_message_part($this->uid, $mime_id, $part);
+                $out = $this->imap->get_message_part($this->uid, $mime_id, $part);
 
                 // remove special chars encoding
                 $trans = array_flip(get_html_translation_table(HTML_ENTITIES));
-                $html_part = strtr($html_part, $trans);
+                $out = strtr($out, $trans);
 
                 // create instance of html2text class
-                $txt = new html2text($html_part);
+                $txt = new html2text($out);
                 $out = $txt->get_text();
-                break;
             }
         }
 
@@ -393,6 +392,17 @@ class rcube_message
                 else if ((($part_mimetype == 'text/plain' || $part_mimetype == 'text/html') && $mail_part->disposition != 'attachment') ||
                     $part_mimetype == 'message/delivery-status' || $part_mimetype == 'message/disposition-notification'
                 ) {
+                    // Allow plugins to handle also this part
+                    $plugin = $this->app->plugins->exec_hook('message_part_structure',
+                        array('object' => $this, 'structure' => $mail_part,
+                            'mimetype' => $part_mimetype, 'recursive' => true));
+
+                    if ($plugin['abort'])
+                        continue;
+
+                    $mail_part = $plugin['structure'];
+                    list($primary_type, $secondary_type) = explode('/', $plugin['mimetype']);
+
                     // add text part if it matches the prefs
                     if (!$this->parse_alternative ||
                         ($secondary_type == 'html' && $this->opt['prefer_html']) ||
@@ -593,10 +603,52 @@ class rcube_message
      */
     public static function unfold_flowed($text)
     {
-        return preg_replace(
-            array('/-- (\r?\n)/',   '/^ /m',  '/(.) \r?\n/',  '/--%SIGEND%(\r?\n)/'),
-            array('--%SIGEND%\\1',  '',       '\\1 ',         '-- \\1'),
-            $text);
+        $text = preg_split('/\r?\n/', $text);
+        $last = -1;
+        $q_level = 0;
+
+        foreach ($text as $idx => $line) {
+            if ($line[0] == '>' && preg_match('/^(>+\s*)/', $line, $regs)) {
+                $q = strlen(str_replace(' ', '', $regs[0]));
+                $line = substr($line, strlen($regs[0]));
+
+                if ($q == $q_level && $line
+                    && isset($text[$last])
+                    && $text[$last][strlen($text[$last])-1] == ' '
+                ) {
+                    $text[$last] .= $line;
+                    unset($text[$idx]);
+                }
+                else {
+                    $last = $idx;
+                }
+            }
+            else {
+                $q = 0;
+                if ($line == '-- ') {
+                    $last = $idx;
+                }
+                else {
+                    // remove space-stuffing
+                    $line = preg_replace('/^\s/', '', $line);
+
+                    if (isset($text[$last]) && $line
+                        && $text[$last] != '-- '
+                        && $text[$last][strlen($text[$last])-1] == ' '
+                    ) {
+                        $text[$last] .= $line;
+                        unset($text[$idx]);
+                    }
+                    else {
+                        $text[$idx] = $line;
+                        $last = $idx;
+                    }
+                }
+            }
+            $q_level = $q;
+        }
+
+        return implode("\r\n", $text);
     }
 
 
@@ -605,17 +657,25 @@ class rcube_message
      */
     public static function format_flowed($text, $length = 72)
     {
-        $out = '';
-    
-        foreach (preg_split('/\r?\n/', trim($text)) as $line) {
-            // don't wrap quoted lines (to avoid wrapping problems)
-            if ($line[0] != '>')
-                $line = rc_wordwrap(rtrim($line), $length - 1, " \r\n");
+        $text = preg_split('/\r?\n/', $text);
 
-            $out .= $line . "\r\n";
+        foreach ($text as $idx => $line) {
+            if ($line != '-- ') {
+                if ($line[0] == '>' && preg_match('/^(>+)/', $line, $regs)) {
+                    $prefix = $regs[0];
+                    $level = strlen($prefix);
+                    $line  = rtrim(substr($line, $level));
+                    $line  = $prefix . rc_wordwrap($line, $length - $level - 2, " \r\n$prefix ");
+                }
+                else if ($line) {
+                    $line = ' ' . rc_wordwrap(rtrim($line), $length - 2, " \r\n ");
+                }
+
+                $text[$idx] = $line;
+            }
         }
-    
-        return $out;
+
+        return implode("\r\n", $text);
     }
 
 }
